@@ -17,12 +17,16 @@
 package javax.el;
 
 import java.beans.FeatureDescriptor;
+import java.util.HashSet;
 import java.util.Iterator;
 import java.util.NoSuchElementException;
+import java.util.Objects;
+import java.util.Set;
 
 public class CompositeELResolver extends ELResolver {
 
     private static final Class<?> SCOPED_ATTRIBUTE_EL_RESOLVER;
+    private static final Set<String> KNOWN_NON_TYPE_CONVERTING_RESOLVERS = new HashSet<>();
     static {
         Class<?> clazz = null;
         try {
@@ -31,36 +35,75 @@ public class CompositeELResolver extends ELResolver {
             // Ignore. This is expected if using the EL stand-alone
         }
         SCOPED_ATTRIBUTE_EL_RESOLVER = clazz;
+
+        // EL API Resolvers
+        KNOWN_NON_TYPE_CONVERTING_RESOLVERS.add(ArrayELResolver.class.getName());
+        KNOWN_NON_TYPE_CONVERTING_RESOLVERS.add(BeanELResolver.class.getName());
+        KNOWN_NON_TYPE_CONVERTING_RESOLVERS.add(BeanNameELResolver.class.getName());
+        KNOWN_NON_TYPE_CONVERTING_RESOLVERS.add(ListELResolver.class.getName());
+        KNOWN_NON_TYPE_CONVERTING_RESOLVERS.add(MapELResolver.class.getName());
+        KNOWN_NON_TYPE_CONVERTING_RESOLVERS.add(ResourceBundleELResolver.class.getName());
+        KNOWN_NON_TYPE_CONVERTING_RESOLVERS.add(StaticFieldELResolver.class.getName());
+        // JSP API Resolvers - referenced by name to avoid creating dependency
+        KNOWN_NON_TYPE_CONVERTING_RESOLVERS.add("jakarta.servlet.jsp.el.ImplicitObjectELResolver");
+        KNOWN_NON_TYPE_CONVERTING_RESOLVERS.add("jakarta.servlet.jsp.el.ScopedAttributeELResolver");
+        // Tomcat internal resolvers - referenced by name to avoid creating dependency
+        KNOWN_NON_TYPE_CONVERTING_RESOLVERS.add("org.apache.el.stream.StreamELResolverImpl");
     }
 
-    private int size;
-
+    private int resolversSize;
     private ELResolver[] resolvers;
 
+    /*
+     * Use a separate array for ELResolvers that might implement type conversion as a performance optimisation. See
+     * https://bz.apache.org/bugzilla/show_bug.cgi?id=68119
+     */
+    private int typeConvertersSize;
+    private ELResolver[] typeConverters;
+
     public CompositeELResolver() {
-        this.size = 0;
-        this.resolvers = new ELResolver[8];
+        resolversSize = 0;
+        resolvers = new ELResolver[8];
+
+        typeConvertersSize = 0;
+        typeConverters = new ELResolver[0];
     }
 
     public void add(ELResolver elResolver) {
-        if (elResolver == null) {
-            throw new NullPointerException();
+        Objects.requireNonNull(elResolver);
+
+        /*
+         * resolversSize should never be larger than resolvers.length. If it ever is, the code will fail when execution
+         * reaches System.arraycopy with an IndexOutOfBoundsException.
+         */
+        if (resolversSize >= resolvers.length) {
+            ELResolver[] nr = new ELResolver[resolversSize * 2];
+            System.arraycopy(resolvers, 0, nr, 0, resolversSize);
+            resolvers = nr;
+        }
+        resolvers[resolversSize++] = elResolver;
+
+        if (KNOWN_NON_TYPE_CONVERTING_RESOLVERS.contains(elResolver.getClass().getName())) {
+            // Performance optimisation. ELResolver known not to perform type conversion
+            return;
         }
 
-        if (this.size >= this.resolvers.length) {
-            ELResolver[] nr = new ELResolver[this.size * 2];
-            System.arraycopy(this.resolvers, 0, nr, 0, this.size);
-            this.resolvers = nr;
+        if (typeConvertersSize == 0) {
+            typeConverters = new ELResolver[1];
+        } else if (typeConvertersSize == typeConverters.length) {
+            ELResolver[] expandedTypeConverters = new ELResolver[typeConvertersSize * 2];
+            System.arraycopy(typeConverters, 0, expandedTypeConverters, 0, typeConvertersSize);
+            typeConverters = expandedTypeConverters;
         }
-        this.resolvers[this.size++] = elResolver;
+        typeConverters[typeConvertersSize++] = elResolver;
     }
 
     @Override
     public Object getValue(ELContext context, Object base, Object property) {
         context.setPropertyResolved(false);
-        int sz = this.size;
+        int sz = resolversSize;
         for (int i = 0; i < sz; i++) {
-            Object result = this.resolvers[i].getValue(context, base, property);
+            Object result = resolvers[i].getValue(context, base, property);
             if (context.isPropertyResolved()) {
                 return result;
             }
@@ -68,14 +111,10 @@ public class CompositeELResolver extends ELResolver {
         return null;
     }
 
-    /**
-     * @since EL 2.2
-     */
     @Override
-    public Object invoke(ELContext context, Object base, Object method,
-            Class<?>[] paramTypes, Object[] params) {
+    public Object invoke(ELContext context, Object base, Object method, Class<?>[] paramTypes, Object[] params) {
         context.setPropertyResolved(false);
-        int sz = this.size;
+        int sz = this.resolversSize;
         for (int i = 0; i < sz; i++) {
             Object obj = this.resolvers[i].invoke(context, base, method, paramTypes, params);
             if (context.isPropertyResolved()) {
@@ -88,7 +127,7 @@ public class CompositeELResolver extends ELResolver {
     @Override
     public Class<?> getType(ELContext context, Object base, Object property) {
         context.setPropertyResolved(false);
-        int sz = this.size;
+        int sz = this.resolversSize;
         for (int i = 0; i < sz; i++) {
             Class<?> type = this.resolvers[i].getType(context, base, property);
             if (context.isPropertyResolved()) {
@@ -111,7 +150,7 @@ public class CompositeELResolver extends ELResolver {
     @Override
     public void setValue(ELContext context, Object base, Object property, Object value) {
         context.setPropertyResolved(false);
-        int sz = this.size;
+        int sz = this.resolversSize;
         for (int i = 0; i < sz; i++) {
             this.resolvers[i].setValue(context, base, property, value);
             if (context.isPropertyResolved()) {
@@ -123,7 +162,7 @@ public class CompositeELResolver extends ELResolver {
     @Override
     public boolean isReadOnly(ELContext context, Object base, Object property) {
         context.setPropertyResolved(false);
-        int sz = this.size;
+        int sz = this.resolversSize;
         for (int i = 0; i < sz; i++) {
             boolean readOnly = this.resolvers[i].isReadOnly(context, base, property);
             if (context.isPropertyResolved()) {
@@ -135,13 +174,13 @@ public class CompositeELResolver extends ELResolver {
 
     @Override
     public Iterator<FeatureDescriptor> getFeatureDescriptors(ELContext context, Object base) {
-        return new FeatureIterator(context, base, this.resolvers, this.size);
+        return new FeatureIterator(context, base, this.resolvers, this.resolversSize);
     }
 
     @Override
     public Class<?> getCommonPropertyType(ELContext context, Object base) {
         Class<?> commonType = null;
-        int sz = this.size;
+        int sz = this.resolversSize;
         for (int i = 0; i < sz; i++) {
             Class<?> type = this.resolvers[i].getCommonPropertyType(context, base);
             if (type != null && (commonType == null || commonType.isAssignableFrom(type))) {
@@ -154,9 +193,9 @@ public class CompositeELResolver extends ELResolver {
     @Override
     public Object convertToType(ELContext context, Object obj, Class<?> type) {
         context.setPropertyResolved(false);
-        int sz = this.size;
+        int sz = typeConvertersSize;
         for (int i = 0; i < sz; i++) {
-            Object result = this.resolvers[i].convertToType(context, obj, type);
+            Object result = this.typeConverters[i].convertToType(context, obj, type);
             if (context.isPropertyResolved()) {
                 return result;
             }
@@ -180,7 +219,7 @@ public class CompositeELResolver extends ELResolver {
 
         private FeatureDescriptor next;
 
-        public FeatureIterator(ELContext context, Object base, ELResolver[] resolvers, int size) {
+        FeatureIterator(ELContext context, Object base, ELResolver[] resolvers, int size) {
             this.context = context;
             this.base = base;
             this.resolvers = resolvers;
@@ -199,9 +238,10 @@ public class CompositeELResolver extends ELResolver {
 
         @Override
         public boolean hasNext() {
-            if (this.next != null)
+            if (this.next != null) {
                 return true;
-            if (this.itr != null){
+            }
+            if (this.itr != null) {
                 while (this.next == null && itr.hasNext()) {
                     this.next = itr.next();
                 }
